import java.applet.Applet;
import java.applet.AudioClip;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.MediaTracker;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.net.URL;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/******** Desert ********
desert class: initializes applet panel
*/
public class Desert extends Applet {
	private JPanel jeepControlsBack, titlePanel, jeepInfoBack;
	private BackGroundCanvas backGroundCanvas;
	private JeepControlsPanel jeepControlsPanel;
	private JeepInformationPanel jeepInfoPanel;
	private ControlsPanel controlsPanel;
	private Image backGroundImage, jeepImage;
	private String backGroundImageFile = "desert.jpg";
	private String jeepImageFile = "4X4.png";
	public	AudioClip jeepSound, petrolSound;
	public Jeep theJeeps[] = new Jeep[9];

	//sets the Desert class as the super class
	public Desert() {
		super();
	}

	/******** init() method ********
	sets the size of the applet, loads the images & sounds
	creates and displays all the panels & jeep objects, calls the newGame method
	*/
	public void init() {
		setJeeps(); //runs the setJeeps method
		setSize(900, 420); //sets the size of the applet

		//loads the two sound files used
		jeepSound = getAudioClip(getDocumentBase(), "jeep.wav");
		petrolSound = getAudioClip(getDocumentBase(), "petrol.wav");

		// loads the images used
		MediaTracker bgTracker = new MediaTracker(this);
		MediaTracker jeepTracker = new MediaTracker(this);
		URL background = getCodeBase();
		URL jeep = getCodeBase();
		backGroundImage = getImage(background, backGroundImageFile);
		jeepImage = getImage(jeep, jeepImageFile);
		bgTracker.addImage(backGroundImage, 0);
		jeepTracker.addImage(jeepImage, 0);
		try {
			bgTracker.waitForID(0);
			jeepTracker.waitForID(0);
		}
		catch (InterruptedException e) {}

		/******** desert Panel ********
		creates a new panel and sets the layout of the panel
		*/
		JPanel desertPanel = new JPanel();
		desertPanel.setLayout(new BorderLayout(0, 0));

		/******** backGroundCanvas ********
		creates a new canvas to display the background and jeep images on
		it then resizes the canvas to the size of the image used for the background
		*/
		backGroundCanvas = new BackGroundCanvas(this, backGroundImage, jeepImage);
		backGroundCanvas.setSize(backGroundImage.getWidth(this), backGroundImage.getHeight(this));

		/******** Title Panel ********
		creates a new panel and display the title of the applet in it
		*/
		titlePanel = new JPanel();
		titlePanel.setLayout(new GridLayout(1, 1));
		titlePanel.add(new JLabel("Exploring the Desert", JLabel.CENTER));

		/******** Jeep Control Panel ********
		creates the panel used for displaying the jeep control panel
		sets the layout used on the panel and add's the jeep control panel
		*/
		jeepControlsBack = new JPanel();
		jeepControlsBack.setLayout(new FlowLayout(FlowLayout.CENTER, 10, 5));
		jeepControlsPanel = new JeepControlsPanel(this);
		jeepControlsBack.add(jeepControlsPanel);

		/******** Jeep Information Panel ********
		creates the panel used for displaying the jeep information panel
		sets the layout used on the panel and add's the jeep information panel
		*/
		jeepInfoBack = new JPanel();
		jeepInfoBack.setLayout(new FlowLayout(FlowLayout.CENTER, 10, 5));
		jeepInfoPanel = new JeepInformationPanel(this);
		jeepInfoBack.add(jeepInfoPanel);

		// Creates the control panel
		controlsPanel = new ControlsPanel(this);

		/******** Constructs the applet ********
		adds the panels created above to the desertPanel
		sets the layout used on the panel and add's the desertPanel
		*/
		JPanel p1 = new JPanel();
		p1.setLayout(new FlowLayout(FlowLayout.CENTER, 10, 10));
		desertPanel.add("Center", backGroundCanvas);
		desertPanel.add("North", titlePanel);
		desertPanel.add("East", jeepControlsBack);
		desertPanel.add("South", controlsPanel);
		desertPanel.add("West", jeepInfoBack);
		p1.add(desertPanel);
		add(p1);

		validate();  //Validates the panel

		newGame();  //calls the newGame method
}

	//creates 9 jeep objects and names them
	public void setJeeps(){
		for(int i = 0; i < 9; i++){
			theJeeps[i] = new Jeep("Jeep " + (i + 1));
		}
	}

	// refreshes the textfields and the canvas back to the default values
    public void newGame() {
        jeepInfoPanel.resetValues();
        jeepInfoPanel.refreshNames();
        backGroundCanvas.refresh();
    }

	/******** rules ********
	used to display the rules the user needs to know to play the game
	*/
	protected String rules() {
		return "This is the rules popup";
	}

	protected void gameWon() {
		String youWin = "Congratulations you have completed the puzzle";
		JOptionPane.showMessageDialog(null, youWin, "Congratulations", JOptionPane.INFORMATION_MESSAGE);
	}

	/******** solution ********
	contains the code to solve the puzzle, called by the user by pressing a button
	*/
	public void solution() {
		int solutionSteps = 9;
        for (int i = 0; i < solutionSteps; i++) {
			doSolutionStep(i);
			try {Thread.sleep(1500);}
			catch (Exception e) {e.printStackTrace();}
		}
	}

	public void doSolutionStep(int step) {
		int y = 0, z = 0;
		switch(step) {
			case 1 : {
				for (int i = z; i < theJeeps.length; i++) {
					forward(theJeeps[i]);
					try {Thread.sleep(1500);}
					catch (Exception e) {e.printStackTrace();}
				}
				for (int x = 1; x < theJeeps.length; x++) {
					theJeeps[y].transfer(theJeeps[x]);
				}
				home(theJeeps[y]);
				y++;
				z++;
				break;
			}
			case 2 : {
				for (int i = z; i < theJeeps.length; i++) {
					forward(theJeeps[i]);
					try {Thread.sleep(1500);}
					catch (Exception e) {e.printStackTrace();}	
				}
				for (int x = 1; x < theJeeps.length; x++) {
					theJeeps[y].transfer(theJeeps[x]);
				}
				home(theJeeps[y]);
				y++;
				z++;
				break;
			}
/*			case 3 : {
				for (int i = z; i < theJeeps.length; i++) {
					forward(theJeeps[i]);
					try {Thread.sleep(1500);}
					catch (Exception e) {e.printStackTrace();}	
					for (int x = 1; x < theJeeps.length; x++) {
						theJeeps[y].transfer(theJeeps[x]);
					}
				}
				home(theJeeps[y]);
				y++;
				z++;
			}
			case 4 : {
				for (int i = z; i < theJeeps.length; i++) {
					forward(theJeeps[i]);
					try {Thread.sleep(1500);}
					catch (Exception e) {e.printStackTrace();}	
					for (int x = 1; x < theJeeps.length; x++) {
						theJeeps[y].transfer(theJeeps[x]);
					}
				}
				home(theJeeps[y]);
				y++;
				z++;
			}
			case  5: {
				for (int i = z; i < theJeeps.length; i++) {
					forward(theJeeps[i]);
					try {Thread.sleep(1500);}
					catch (Exception e) {e.printStackTrace();}	
					for (int x = 1; x < theJeeps.length; x++) {
						theJeeps[y].transfer(theJeeps[x]);
					}
				}
				home(theJeeps[y]);
				y++;
				z++;
			}
			case 6 : {
				for (int i = z; i < theJeeps.length; i++) {
					forward(theJeeps[i]);
					try {Thread.sleep(1500);}
					catch (Exception e) {e.printStackTrace();}	
					for (int x = 1; x < theJeeps.length; x++) {
						theJeeps[y].transfer(theJeeps[x]);
					}
				}
				home(theJeeps[y]);
				y++;
				z++;
			}
			case 7 : {
				for (int i = z; i < theJeeps.length; i++) {
					forward(theJeeps[i]);
					try {Thread.sleep(1500);}
					catch (Exception e) {e.printStackTrace();}	
					for (int x = 1; x < theJeeps.length; x++) {
						theJeeps[y].transfer(theJeeps[x]);
					}
				}
				home(theJeeps[y]);
				y++;
				z++;
			}
			case 8 : {
				for (int i = z; i < theJeeps.length; i++) {
					forward(theJeeps[i]);
					try {Thread.sleep(1500);}
					catch (Exception e) {e.printStackTrace();}	
					for (int x = 1; x < theJeeps.length; x++) {
						theJeeps[y].transfer(theJeeps[x]);
					}
				}
				home(theJeeps[y]);
				y++;
				z++;
			}
			case 9 : {
				for (int i = z; i < theJeeps.length; i++) {
					forward(theJeeps[i]);
					try {Thread.sleep(1500);}
					catch (Exception e) {e.printStackTrace();}
					for (int x = 1; x < theJeeps.length; x++) {
						theJeeps[y].transfer(theJeeps[x]);
					}
				}
				home(theJeeps[y]);
				y++;
				z++;
			}
*/		}
	}

	/******** home ********
	moves the jeeps back to the start of the course
	*/
	protected void home(Jeep selectedJeep){
		String homeError = "You are already home";  //only displayed if the jeep is already at the start

		if (selectedJeep.home()) {
			JOptionPane.showMessageDialog(null, homeError, "Error", JOptionPane.ERROR_MESSAGE);
		}
		else {
			selectedJeep.home();				//calls the home method for the selected jeep
			backGroundCanvas.refresh();			//refreshes the images
			jeepInfoPanel.refreshDistances();	//resets the distance textfields
			jeepInfoPanel.refreshPetrol();		//resets the petrol textfields
			jeepSound.play();					//plays the forward.wav file
		}
	}

	/******** forward ********
	moves the jeeps forward
	*/
	protected void forward(Jeep selectedJeep) {
		String moveError = "You can't move any further forward, you don't have enough petrol to return home with!",
			   noPetrol = "You can't move forward, you don't have any petrol left!";

		if(selectedJeep.forward()){
			backGroundCanvas.refresh();			//refreshes the images
			jeepInfoPanel.refreshDistances();	//resets the distance textfields
			jeepInfoPanel.refreshPetrol();		//resets the petrol textfields
			jeepSound.play();					//plays the forward.wav file
		}
		else {
			if (selectedJeep.petrol == 0) {
				JOptionPane.showMessageDialog(null, noPetrol, "Error", JOptionPane.ERROR_MESSAGE);
			}
			else {
				JOptionPane.showMessageDialog(null, moveError, "Error", JOptionPane.ERROR_MESSAGE);
			}

		}
	}

	/******** transfer ********
	transfers petrol between jeeps
	*/
	protected void transfer(Jeep transferFromJeep, Jeep transferToJeep) {
		String transferError = "You can't do that";

		if (transferFromJeep.transfer(transferToJeep)) {
			transferFromJeep.transfer(transferToJeep);	//?
			jeepInfoPanel.refreshPetrol();				//refreshes the petrol textfields
			petrolSound.play();
		}
		else {
			JOptionPane.showMessageDialog(null, transferError, "Error", JOptionPane.ERROR_MESSAGE);
		}

	}
}  //end of the desert class

/******** JEEP CONTROLS PANEL ********
contains the code used to create the jeep control panel
add's buttons and actionlisteners
*/
final class JeepControlsPanel extends JPanel implements ActionListener {
	private JComboBox jeepsCombo, transferToCombo, transferFromCombo;
	private JPanel jeepControlsPanel, jeepTransferPanel;
	private JButton bForward, bHome, bTransfer;
	private Desert desert;

	// constructor
	JeepControlsPanel(Desert desert) {
		this.desert = desert;

		setLayout(new GridLayout(2, 1));

		// Jeep controls
		jeepControlsPanel = new JPanel();
		jeepControlsPanel.setLayout(new GridLayout(5, 2, 0, 5));

		jeepControlsPanel.add(new JLabel("", JLabel.LEFT));
		jeepControlsPanel.add(new JLabel("", JLabel.LEFT));
		jeepControlsPanel.add(new JLabel("Move", JLabel.LEFT));
		jeepControlsPanel.add(jeepsCombo = new JComboBox(desert.theJeeps));
		jeepControlsPanel.add(new JLabel("", JLabel.LEFT));
		jeepControlsPanel.add(new JLabel("", JLabel.LEFT));
		jeepControlsPanel.add(bForward = new JButton("Forward"));
		bForward.addActionListener(this);
		bForward.setToolTipText("Click here to move the car forward");
		jeepControlsPanel.add(bHome = new JButton("Home"));
		bHome.addActionListener(this);
		bHome.setToolTipText("Click here to send the car home");
		jeepControlsPanel.add(new JLabel("", JLabel.LEFT));
		jeepControlsPanel.add(new JLabel("", JLabel.LEFT));
		jeepControlsPanel.validate();

		// Transfer panel
		jeepTransferPanel = new JPanel();
		jeepTransferPanel.setLayout(new GridLayout(4, 2, 0, 5));

		jeepTransferPanel.add(new JLabel("Transfer Petrol", JLabel.LEFT));
		jeepTransferPanel.add(new JLabel("", JLabel.LEFT));
		jeepTransferPanel.add(new JLabel("From", JLabel.LEFT));

		jeepTransferPanel.add(transferFromCombo = new JComboBox(desert.theJeeps));

		jeepTransferPanel.add(new JLabel("To", JLabel.LEFT));

		jeepTransferPanel.add(transferToCombo = new JComboBox(desert.theJeeps));

		jeepTransferPanel.add(bTransfer = new JButton("Transfer"));
		bTransfer.addActionListener(this);
		bTransfer.setToolTipText("Click here to transfer petrol between the jeeps");

		jeepTransferPanel.validate();

		// construct panel

		add(jeepControlsPanel);
		add(jeepTransferPanel);

		validate();
}


	public void actionPerformed (ActionEvent e) {
		if (e.getSource() == bTransfer) {
			desert.transfer((Jeep)transferFromCombo.getSelectedItem(), (Jeep)transferToCombo.getSelectedItem());
		}
		if (e.getSource() == bForward) {
			desert.forward((Jeep)jeepsCombo.getSelectedItem());
		}
		if (e.getSource() == bHome) {
			int userResponse = JOptionPane.showConfirmDialog(null, "Are you sure you want to take the jeep home?", null, JOptionPane.YES_NO_OPTION);
			if (userResponse == JOptionPane.YES_OPTION) {desert.home((Jeep)jeepsCombo.getSelectedItem());}
			if (userResponse == JOptionPane.NO_OPTION) {};
		}
	}
}

/******** BOARD CANVAS ********
draws the board using double buffer,
passes canvas mouse events to desert class
*/
final class BackGroundCanvas extends Canvas {
	private Image canvas, jeep, background;
	private Desert desert;
	private int bwidth, bheight, jheight;

	// constructor
	BackGroundCanvas(Desert desert, Image background, Image jeep) {
		this.desert = desert;
		this.background = background;
		this.jeep = jeep;
	}

	public void refresh() {
		bwidth = background.getWidth(this);
		bheight = background.getHeight(this);
		jheight = jeep.getHeight(this);

		canvas = createImage(bwidth, bheight);
		canvas.getGraphics().drawImage(background, 0, 0, this);

		for (int i = 0; i < desert.theJeeps.length; i++) {
			drawJeeps(canvas.getGraphics(), jeep, bwidth - desert.theJeeps[i].distance - 60, (45 + (i * (jheight + 4))));
		}
		repaint();
	}

	private void drawJeeps(Graphics g, Image jeep, int x, int y) {
		g.drawImage(jeep, x, y, this);
	}

	public void paint(Graphics g) {
		g.drawImage(canvas, 0, 0, this);
	}
}

/******** JEEP INFORMATION PANEL ********
information about the jeeps such as how much petrol they have redeserting,
the distance they have traveled.
*/
final class JeepInformationPanel extends JPanel {
	private JPanel jeepDistancePanel, jeepPetrolPanel, jeepNamePanel;
	private JTextField distanceFields[] = new JTextField[9];
	private JTextField petrolFields[] = new JTextField[9];
	private JTextField nameFields[] = new JTextField[9];
	private Desert desert;

	JeepInformationPanel(Desert desert) {
		this.desert = desert;

		// create jeep Distance panel
		jeepNamePanel = new JPanel();
		jeepNamePanel.setLayout(new GridLayout(10, 1, 0, 14));
		jeepNamePanel.add(new JLabel("Jeep Name", JLabel.LEFT));

		// create jeep Distance panel
		jeepDistancePanel = new JPanel();
		jeepDistancePanel.setLayout(new GridLayout(10, 1, 0, 14));
		jeepDistancePanel.add(new JLabel("Distance", JLabel.LEFT));

		// create jeep Petrol panel
		jeepPetrolPanel = new JPanel();
		jeepPetrolPanel.setLayout(new GridLayout (10, 1, 0, 14));
		jeepPetrolPanel.add(new JLabel("Petrol", JLabel.LEFT));

		for (int i = 0; i < desert.theJeeps.length; i++) {
			petrolFields[i] = new JTextField(2);
			distanceFields[i] = new JTextField(2);
			nameFields[i] = new JTextField(2);
			petrolFields[i].setEditable(false);
			distanceFields[i].setEditable(false);
			nameFields[i].setEditable(false);
			jeepPetrolPanel.add(petrolFields[i]);
			jeepDistancePanel.add(distanceFields[i]);
			jeepNamePanel.add(nameFields[i]);
		}

		jeepPetrolPanel.validate();
		jeepDistancePanel.validate();
		jeepNamePanel.validate();

		add(jeepNamePanel);
		add(jeepDistancePanel);
		add(jeepPetrolPanel);
		validate() ;
	}

	public void refreshPetrol() {
		for (int i = 0; i < desert.theJeeps.length; i++) {
			petrolFields[i].setText(String.valueOf(desert.theJeeps[i].petrol));
		}
	}

	public void refreshDistances() {
		for (int i = 0; i < desert.theJeeps.length; i++) {
			distanceFields[i].setText(String.valueOf(desert.theJeeps[i].distance));
		}
	}

	public void refreshNames() {
		for (int i = 0; i < desert.theJeeps.length; i++) {
			nameFields[i].setText(String.valueOf(desert.theJeeps[i].name));
		}
	}

	public void resetValues() {
		for (int i = 0; i < desert.theJeeps.length; i++) {
			petrolFields[i].setText(String.valueOf(desert.theJeeps[i].setPetrol()));
			distanceFields[i].setText(String.valueOf(desert.theJeeps[i].setDistance()));
		}
	}
}

/******** CONTROLS PANEL ********
UI components for status messages and move counter
*/
final class ControlsPanel extends JPanel implements ActionListener {
	private JButton bReset, bRules, bSolution;
	private JPanel buttonsPanel;
	private Desert desert;

	// constructor
	ControlsPanel(Desert desert) {
		this.desert = desert;

		buttonsPanel = new JPanel();
		buttonsPanel.setLayout(new GridLayout(1, 3, 5, 0));
		buttonsPanel.add(bReset = new JButton("Reset"));
		bReset.setToolTipText("Click here to reset the applet");
		bReset.addActionListener(this);

		buttonsPanel.add(bRules = new JButton("Rules"));
		bRules.addActionListener(this);
		bRules.setToolTipText("Click here to see the rules");
		
		buttonsPanel.add(bSolution = new JButton("Solution"));
		bSolution.addActionListener(this);
		bSolution.setToolTipText("Click here to view the solution");
		
		buttonsPanel.validate();
		add(buttonsPanel);

		validate();
	}

	public void actionPerformed (ActionEvent e) {
		if (e.getSource() == bRules) {
			JOptionPane.showMessageDialog(null, desert.rules(), "Rules", JOptionPane.INFORMATION_MESSAGE);
		}
		if (e.getSource() == bReset) {
//			int userResponse = JOptionPane.showConfirmDialog(null, "Are you sure you want to reset?");
			int userResponse = JOptionPane.showConfirmDialog(null, "Are you sure you want to reset?", null, JOptionPane.YES_NO_OPTION);
			if (userResponse == JOptionPane.YES_OPTION) {desert.newGame();}
			if (userResponse == JOptionPane.NO_OPTION) {};
		}
		if (e.getSource() == bSolution) {
			int userResponse = JOptionPane.showConfirmDialog(null, "Are you sure you want to see the solution?", null, JOptionPane.YES_NO_OPTION);
			if (userResponse == JOptionPane.YES_OPTION) {desert.solution();}
			if (userResponse == JOptionPane.NO_OPTION) {};
		}
	}
}

/******** JEEP ********
Methods use to control the jeeps
*/
final class Jeep {
	public String name;
	public int petrol, distance;
	private Desert desert;

	protected Jeep(String jeepName) {
		petrol = 0;
		distance = 0;
		name = jeepName;
	}

	private boolean canGoForward() {
		if ((distance / 40) <= (petrol - 1)) {
			return true;
		}
		else {return false;}
	}

    protected boolean forward() {
        this.desert = desert;

        if (won()) {
            desert.gameWon();
            return false;
        }
        else {
            if (canGoForward()) {
                distance += 40;
                petrol -= 1;
                return true;
            }
            else {return false;}
        }
    }

    private boolean won() {
        if (distance == (9 * 40)) {
            return true;
        }
        else {return false;}
    }

	protected boolean home() {
		if (distance == 0) {
			return true;
		}
		else {
			if (distance > 0 ) {
				if ((distance / 40) == petrol) {
					petrol = 0;
				}
				else {
					petrol -= (distance / 40);

				}
				distance = 0;

			}
			return false;
		}

	}

	private boolean canTransfer(Jeep transferToJeep) {
			if ((distance == transferToJeep.distance) && (canGoForward() && (transferToJeep.petrol < 10))) {
				return true;
			}
			else {return false;}
	}

	protected boolean transfer(Jeep transferToJeep) {
		if (canTransfer(transferToJeep)) {
			petrol -= 1;
			transferToJeep.petrol += 1;
			return true;
		}
		else {return false;}
	}

	public String toString() {
		return name;
	}

	public int setPetrol() {
		return petrol = 10;
	}

	public int setDistance() {
		return distance = 0;
	}
}